# Copyright 2020 Ben Kehoe
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utility for determining what AWS account and identity you're using."""

from __future__ import print_function

import argparse
from collections import namedtuple
import json
import sys
import os
import traceback

import botocore
import botocore.session
from botocore.exceptions import ClientError

__version__ = '1.2.0'

WhoamiInfo = namedtuple('WhoamiInfo', [
    'Account',
    'AccountAliases',
    'Arn',
    'Type',
    'Name',
    'RoleSessionName',
    'UserId',
    'Region',
    'SSOPermissionSet',
])

DESCRIPTION = """\
Show what AWS account and identity you're using.
Formats the output of sts.GetCallerIdentity nicely,
and also gets your account alias (if you're allowed)
"""

def main():
    parser = argparse.ArgumentParser(description=DESCRIPTION)

    parser.add_argument('--profile', help="AWS profile to use")

    parser.add_argument('--json', action='store_true', help="Output as JSON")

    parser.add_argument('--version', action='store_true')

    parser.add_argument('--debug', action='store_true')

    args = parser.parse_args()

    if args.version:
        print(__version__)
        parser.exit()

    try:
        session = botocore.session.Session(profile=args.profile)

        disable_account_alias = os.environ.get('AWS_WHOAMI_DISABLE_ACCOUNT_ALIAS', '')
        if disable_account_alias.lower() in ['', '0', 'false']:
            disable_account_alias = False
        elif disable_account_alias.lower() in ['1', 'true']:
            disable_account_alias = True
        else:
            disable_account_alias = disable_account_alias.split(',')

        whoami_info = whoami(session=session, disable_account_alias=disable_account_alias)

        if args.json:
            print(json.dumps(whoami_info._asdict()))
        else:
            print(format_whoami(whoami_info))
    except Exception as e:
        if args.debug:
            traceback.print_exc()
        err_cls = type(e)
        err_cls_str = err_cls.__name__
        if err_cls.__module__ != 'builtins':
            err_cls_str = '{}.{}'.format(err_cls.__module__, err_cls_str)
        sys.stderr.write('ERROR [{}]: {}\n'.format(err_cls_str, e))
        sys.exit(1)

def format_whoami(whoami_info):
    lines = []
    lines.append(('Account: ', whoami_info.Account))
    for alias in whoami_info.AccountAliases:
        lines.append(('', alias))
    lines.append(('Region: ', whoami_info.Region))
    if whoami_info.SSOPermissionSet:
        lines.append(('AWS SSO: ', whoami_info.SSOPermissionSet))
    else:
        type_str = ''.join(p[0].upper() + p[1:] for p in whoami_info.Type.split('-'))
        lines.append(('{}: '.format(type_str), whoami_info.Name))
    if whoami_info.RoleSessionName:
        lines.append(('RoleSessionName: ', whoami_info.RoleSessionName))
    lines.append(('UserId: ', whoami_info.UserId))
    lines.append(('Arn: ', whoami_info.Arn))
    max_len = max(len(l[0]) for l in lines)
    return '\n'.join("{}{}".format(l[0].ljust(max_len), l[1]) for l in lines)

def whoami(session=None, disable_account_alias=False):
    """Return a WhoamiInfo namedtuple.

    Args:
        session: An optional boto3 or botocore Session
        disable_account_alias (bool): Disable checking the account alias

    Returns:
        WhoamiInfo: Data on the current IAM principal, account, and region.

    """
    if session is None:
        session = botocore.session.get_session()
    elif hasattr(session, '_session'): # allow boto3 Session as well
        session = session._session

    data = {}
    data['Region'] = session.get_config_variable('region')

    response = session.create_client('sts').get_caller_identity()

    for field in ['Account', 'Arn', 'UserId']:
        data[field] = response[field]

    data['Type'], name = data['Arn'].rsplit(':', 1)[1].split('/',1)

    if data['Type'] == 'assumed-role':
        data['Name'], data['RoleSessionName'] = name.rsplit('/', 1)
    else:
        data['Name'] = name
        data['RoleSessionName'] = None

    if data['Type'] == 'assumed-role' and data['Name'].startswith('AWSReservedSSO'):
        try:
            # format is AWSReservedSSO_{permission-set}_{random-tag}
            data['SSOPermissionSet'] = data['Name'].split('_', 1)[1].rsplit('_', 1)[0]
        except Exception as e:
            data['SSOPermissionSet'] = None
    else:
        data['SSOPermissionSet'] = None

    data['AccountAliases'] = []
    if not isinstance(disable_account_alias, bool):
        for value in disable_account_alias:
            if data['Account'].startswith(value) or data['Account'].endswith(value):
                disable_account_alias = True
                break
            fields = ['Name', 'Arn', 'RoleSessionName']
            if any(value == data[field] for field in fields):
                disable_account_alias = True
                break
    if not disable_account_alias:
        try:
            #pedantry
            paginator = session.create_client('iam').get_paginator('list_account_aliases')
            for response in paginator.paginate():
                data['AccountAliases'].extend(response['AccountAliases'])
        except ClientError as e:
            if e.response.get('Error', {}).get('Code') != 'AccessDenied':
                raise

    return WhoamiInfo(**data)

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

modules = \
['aws_whoami']
install_requires = \
['boto3']

entry_points = \
{'console_scripts': ['aws-whoami = aws_whoami:main']}

setup_kwargs = {
    'name': 'aws-whoami',
    'version': '1.2.0',
    'description': "A tool and library for determining what AWS account and identity you're using",
    'long_description': "# aws-whoami\n**Show what AWS account and identity you're using**\n\nYou should know about [`aws sts get-caller-identity`](https://docs.aws.amazon.com/cli/latest/reference/sts/get-caller-identity.html),\nwhich sensibly returns the identity of the caller. But even with `--output table`, I find this a bit lacking.\nThat ARN is a lot to visually parse, it doesn't tell you what region your credentials are configured for,\nand I am not very good at remembering AWS account numbers. `aws-whoami` makes it better.\n\n```\n$ aws-whoami\nAccount:         123456789012\n                 my-account-alias\nRegion:          us-east-2\nAssumedRole:     MY-ROLE\nRoleSessionName: ben\nUserId:          SOMEOPAQUEID:ben\nArn:             arn:aws:sts::123456789012:assumed-role/MY-ROLE/ben\n```\n\nNote: if you don't have permissions to [iam:ListAccountAliases](https://docs.aws.amazon.com/IAM/latest/APIReference/API_ListAccountAliases.html),\nyour account alias won't appear. See below for disabling this check if getting a permission denied on this call raises flags in your organization.\n\n## Install\n\nI recommend you install `aws-whoami` with [`pipx`](https://pipxproject.github.io/pipx/), which installs the tool in an isolated virtualenv while linking the script you need.\n\n```bash\n# with pipx\npipx install aws-whoami\n\n# without pipx\npython -m pip install --user aws-whoami\n```\n\nIf you don't want to install it, the `aws_whoami.py` file can be used on its own, with only a dependency on `botocore` (which comes with `boto3`).\n\n## Options\n\n`aws-whoami` uses [`boto3`](boto3.amazonaws.com/v1/documentation/api/latest/index.html), so it'll pick up your credentials in [the normal ways](https://docs.aws.amazon.com/cli/latest/userguide/cli-chap-configure.html#config-settings-and-precedence),\nincluding with the `--profile` parameter.\n\nIf you'd like the output as a JSON object, that's the `--json` flag.\nThe output is the `WhoamiInfo` object (see below) as a JSON object.\n\nTo full disable account alias checking, set the environment variable `AWS_WHOAMI_DISABLE_ACCOUNT_ALIAS` to `true`.\nTo selectively disable it, you can also set it to a comma-separated list of values that will be matched against the following:\n* The beginning or end of the account number\n* The principal Name or ARN\n* The role session name\n\n## As a library\n\nThe library has a `whoami()` function, which optionally takes a `Session` (either `boto3` or `botocore`), and returns a `WhoamiInfo` namedtuple.\n\nThe fields of `WhoamiInfo` are:\n* `Account`\n* `AccountAliases` (NOTE: this is a list)\n* `Arn`\n* `Type`\n* `Name`\n* `RoleSessionName`\n* `UserId`\n* `Region`\n* `SSOPermissionSet`\n\n`Type`, `Name`, and `RoleSessionName` (and `SSOPermissionSet`) are split from the ARN for convenience.\n`RoleSessionName` is `None` for IAM users.\n\n`SSOPermissionSet` is set if the assumed role name conforms to the format `AWSReservedSSO_{permission-set}_{random-tag}`.\n\nTo disable the account alias check, pass `disable_account_alias=True` to `whoami()`.\nNote that the `AccountAliases` field will then be an empty list, not `None`.\n\n`format_whoami()` takes a `WhoamiInfo` object and returns the formatted string used for display.\n",
    'author': 'Ben Kehoe',
    'author_email': 'ben@kehoe.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/benkehoe/aws-whoami',
    'py_modules': modules,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=2.7, !=3.0.*, !=3.1.*, !=3.2.*',
}


setup(**setup_kwargs)
